@@ -10,6 +10,7 @@ import functools
 import logging
 import os
 import pathlib
+import socket
 import subprocess
 import sys
 import threading
@@ -23,6 +24,7 @@ from typing import (
     Iterable,
     Iterator,
     List,
+    Literal,
     Mapping,
     Optional,
     Sequence,
@@ -31,11 +33,12 @@ from typing import (
     Union,
     cast,
 )
+from urllib import parse as urllib_parse
 
 import httpx
 import requests
 from typing_extensions import ParamSpec
-from urllib3.util import Retry
+from urllib3.util import Retry  # type: ignore[import-untyped]
 
 from langsmith import schemas as ls_schemas
 
@@ -50,6 +53,10 @@ class LangSmithAPIError(LangSmithError):
     """Internal server error while communicating with LangSmith."""
 
 
+class LangSmithRequestTimeout(LangSmithError):
+    """Client took too long to send request body."""
+
+
 class LangSmithUserError(LangSmithError):
     """User error caused an exception when communicating with LangSmith."""
 
@@ -74,7 +81,18 @@ class LangSmithConnectionError(LangSmithError):
     """Couldn't connect to the LangSmith API."""
 
 
-def tracing_is_enabled(ctx: Optional[dict] = None) -> bool:
+## Warning classes
+
+
+class LangSmithWarning(UserWarning):
+    """Base class for warnings."""
+
+
+class LangSmithMissingAPIKeyWarning(LangSmithWarning):
+    """Warning for missing API key."""
+
+
+def tracing_is_enabled(ctx: Optional[dict] = None) -> Union[bool, Literal["local"]]:
     """Return True if tracing is enabled."""
     from langsmith.run_helpers import get_current_run_tree, get_tracing_context
 
@@ -133,9 +151,6 @@ def raise_for_status_with_text(
     except requests.HTTPError as e:
         raise requests.HTTPError(str(e), response.text) from e  # type: ignore[call-arg]
 
-    except httpx.HTTPError as e:
-        raise httpx.HTTPError(str(e), response.text) from e  # type: ignore[call-arg]
-
 
 def get_enum_value(enu: Union[enum.Enum, str]) -> str:
     """Get the value of a string enum."""
@@ -339,6 +354,7 @@ def is_base_message_like(obj: object) -> bool:
     )
 
 
+@functools.lru_cache(maxsize=100)
 def get_env_var(
     name: str,
     default: Optional[str] = None,
@@ -366,6 +382,7 @@ def get_env_var(
     return default
 
 
+@functools.lru_cache(maxsize=1)
 def get_tracer_project(return_default_value=True) -> Optional[str]:
     """Get the project name for a LangSmith tracer."""
     return os.environ.get(
@@ -476,6 +493,10 @@ def with_cache(
             "vcrpy is required to use caching. Install with:"
             'pip install -U "langsmith[vcr]"'
         )
+    # Fix concurrency issue in vcrpy's patching
+    from langsmith._internal import _patch as patch_urllib3
+
+    patch_urllib3.patch_urllib3()
 
     def _filter_request_headers(request: Any) -> Any:
         if ignore_hosts and any(request.url.startswith(host) for host in ignore_hosts):
@@ -661,7 +682,7 @@ class ContextThreadPoolExecutor(ThreadPoolExecutor):
     ) -> Iterator[T]:
         """Return an iterator equivalent to stdlib map.
 
-        Each function will receive it's own copy of the context from the parent thread.
+        Each function will receive its own copy of the context from the parent thread.
 
         Args:
             fn: A callable that will take as many arguments as there are
@@ -715,3 +736,74 @@ def get_api_key(api_key: Optional[str]) -> Optional[str]:
     if api_key_ is None or not api_key_.strip():
         return None
     return api_key_.strip().strip('"').strip("'")
+
+
+def _is_localhost(url: str) -> bool:
+    """Check if the URL is localhost.
+
+    Parameters
+    ----------
+    url : str
+        The URL to check.
+
+    Returns:
+    -------
+    bool
+        True if the URL is localhost, False otherwise.
+    """
+    try:
+        netloc = urllib_parse.urlsplit(url).netloc.split(":")[0]
+        ip = socket.gethostbyname(netloc)
+        return ip == "127.0.0.1" or ip.startswith("0.0.0.0") or ip.startswith("::")
+    except socket.gaierror:
+        return False
+
+
+@functools.lru_cache(maxsize=2)
+def get_host_url(web_url: Optional[str], api_url: str):
+    """Get the host URL based on the web URL or API URL."""
+    if web_url:
+        return web_url
+    parsed_url = urllib_parse.urlparse(api_url)
+    if _is_localhost(api_url):
+        link = "http://localhost"
+    elif str(parsed_url.path).endswith("/api"):
+        new_path = str(parsed_url.path).rsplit("/api", 1)[0]
+        link = urllib_parse.urlunparse(parsed_url._replace(path=new_path))
+    elif str(parsed_url.netloc).startswith("eu."):
+        link = "https://eu.smith.langchain.com"
+    elif str(parsed_url.netloc).startswith("dev."):
+        link = "https://dev.smith.langchain.com"
+    else:
+        link = "https://smith.langchain.com"
+    return link
+
+
+def _get_function_name(fn: Callable, depth: int = 0) -> str:
+    if depth > 2 or not callable(fn):
+        return str(fn)
+
+    if hasattr(fn, "__name__"):
+        return fn.__name__
+
+    if isinstance(fn, functools.partial):
+        return _get_function_name(fn.func, depth + 1)
+
+    if hasattr(fn, "__call__"):
+        if hasattr(fn, "__class__") and hasattr(fn.__class__, "__name__"):
+            return fn.__class__.__name__
+        return _get_function_name(fn.__call__, depth + 1)
+
+    return str(fn)
+
+
+def is_truish(val: Any) -> bool:
+    """Check if the value is truish.
+
+    Args:
+        val (Any): The value to check.
+
+    Returns:
+        bool: True if the value is truish, False otherwise.
+    """
+    return val is True or val == "true" or val == "True" or val == "TRUE" or val == "1"
